@@ -583,7 +583,7 @@ winCreateWindowsTopLevelWindow(WindowPtr pWin)
        make sure the window actually ends up somewhere where it will be visible
 
        To handle arrangements of monitors which form a non-rectangular virtual
-       desktop, check if the window will end up with it's top-left corner on any
+       desktop, check if the window will end up with its top-left corner on any
        monitor
     */
     if (pWin->drawable.class != InputOnly) {
@@ -643,7 +643,7 @@ winCreateWindowsTopLevelWindow(WindowPtr pWin)
 
     /*
        Calculate the window coordinates containing the requested client area,
-       being careful to preseve CW_USEDEFAULT
+       being careful to preserve CW_USEDEFAULT
      */
     rc.top = (iY != CW_USEDEFAULT) ? iY : 0;
     rc.left = (iX != CW_USEDEFAULT) ? iX : 0;
@@ -955,7 +955,7 @@ winReorderWindowsMultiWindow(void)
     WindowPtr pWin = NULL;
     WindowPtr pWinSib = NULL;
     XID vlist[2];
-    static Bool fRestacking = FALSE;    /* Avoid recusive calls to this function */
+    static Bool fRestacking = FALSE; /* Avoid recursive calls to this function */
     DWORD dwCurrentProcessID = GetCurrentProcessId();
     DWORD dwWindowProcessID = 0;
 
@@ -964,7 +964,7 @@ winReorderWindowsMultiWindow(void)
 #endif
 
     if (fRestacking) {
-        /* It is a recusive call so immediately exit */
+        /* It is a recursive call so immediately exit */
 #if CYGWINDOWING_DEBUG
         ErrorF("winReorderWindowsMultiWindow - "
                "exit because fRestacking == TRUE\n");
@@ -1174,7 +1174,7 @@ winAdjustXWindow(WindowPtr pWin, HWND hwnd)
 }
 
 /*
-  Helper function
+  Helper function for creating a DIB to back a pixmap
  */
 static HBITMAP winCreateDIB(ScreenPtr pScreen, int width, int height, int bpp, void **ppvBits, BITMAPINFOHEADER **ppbmih)
 {
@@ -1282,7 +1282,7 @@ winCreatePixmapMultiwindow(ScreenPtr pScreen, int width, int height, int depth,
     pPixmap->devKind = paddedwidth;
     pPixmap->refcnt = 1;
     pPixmap->devPrivate.ptr = NULL; // later set to pbBits
-    pPixmap->master_pixmap = NULL;
+    pPixmap->primary_pixmap = NULL;
 #ifdef COMPOSITE
     pPixmap->screen_x = 0;
     pPixmap->screen_y = 0;
@@ -1304,6 +1304,7 @@ winCreatePixmapMultiwindow(ScreenPtr pScreen, int width, int height, int depth,
 
     /* Create a DIB for the pixmap */
     pPixmapPriv->hBitmap = winCreateDIB(pScreen, width, height, bpp, &pPixmapPriv->pbBits, &pPixmapPriv->pbmih);
+    pPixmapPriv->owned = TRUE;
 
     winDebug("winCreatePixmap: pPixmap %p HBITMAP %p pBMIH %p pBits %p\n", pPixmap, pPixmapPriv->hBitmap, pPixmapPriv->pbmih, pPixmapPriv->pbBits);
     /* XXX: so why do we need this in privates ??? */
@@ -1335,6 +1336,10 @@ winDestroyPixmapMultiwindow(PixmapPtr pPixmap)
     /* Get a handle to the pixmap privates */
     pPixmapPriv = winGetPixmapPriv(pPixmap);
 
+    /* Nothing to do if we don't own the DIB */
+    if (!pPixmapPriv->owned)
+        return TRUE;
+
     /* Free GDI bitmap */
     if (pPixmapPriv->hBitmap)
         DeleteObject(pPixmapPriv->hBitmap);
@@ -1361,7 +1366,7 @@ winModifyPixmapHeaderMultiwindow(PixmapPtr pPixmap,
                                  int bitsPerPixel, int devKind, void *pPixData)
 {
     int i;
-    winPrivPixmapPtr pPixmapPriv = NULL;
+    winPrivPixmapPtr pPixmapPriv = winGetPixmapPriv(pPixmap);
     Bool fResult;
 
     /* reinitialize everything */
@@ -1391,28 +1396,25 @@ winModifyPixmapHeaderMultiwindow(PixmapPtr pPixmap,
         if (pScreenInfo->pfb == pPixData)
             {
                 /* and initialize pixmap privates from screen privates */
-                pPixmapPriv = winGetPixmapPriv(pPixmap);
-
                 pPixmapPriv->hBitmap = pScreenPriv->hbmpShadow;
                 pPixmapPriv->pbBits = pScreenInfo->pfb;
                 pPixmapPriv->pbmih = pScreenPriv->pbmih;
 
-                /* XXX: need to mark these not to get released in DestroyPixmap */
+                /* mark these not to get released by DestroyPixmap */
+                pPixmapPriv->owned = FALSE;
 
                 return TRUE;
             }
     }
 
     /* Otherwise, since creating a DIBSection from arbitrary memory is not
-     * possible, fallback to normal.  Later we can create a DIBSection with a
-     * copy of the bits, if needed. */
-    {
-        pPixmapPriv = winGetPixmapPriv(pPixmap);
-
-        pPixmapPriv->hBitmap = 0;
-        pPixmapPriv->pbBits = 0;
-        pPixmapPriv->pbmih = 0;
-    }
+     * possible, fallback to normal.  If needed, we can create a DIBSection with
+     * a copy of the bits later (see comment about a potential slow-path in
+     * winBltExposedWindowRegionShadowGDI()). */
+    pPixmapPriv->hBitmap = 0;
+    pPixmapPriv->pbBits = 0;
+    pPixmapPriv->pbmih = 0;
+    pPixmapPriv->owned = FALSE;
 
     winDebug("winModifyPixmapHeaderMultiwindow: falling back\n");
 
